/// <reference types="react" />
import { Event, EventMatcher, Filter, Listener, ListenerConfig } from "polyrhythm";
import { Subscription } from "rxjs";
export { trigger } from "polyrhythm";
interface ListenerConfigWithDeps extends ListenerConfig {
    deps?: Array<any>;
}
/** Sets up a channel - a domain for events. It's not always necessary or
 * advantageous to use multiple channels - a single channel may suffice, and
 * different slices of it managed via namespacing.
 */
export declare const ChannelContext: import("react").Context<import("polyrhythm").Channel>;
/**
 * An alternative to the top-level static imports of useListener, trigger, etc.
 * that implicitly use the default channel. Uses a channel provided by a ChannelContext,
 * or falls back to the default channel.
 * @example: const { trigger, useListener } = useChannel();
 */
export declare const useChannel: () => {
    channel: import("polyrhythm").Channel;
    trigger(type: string, payload?: any): void;
    query(matcher: EventMatcher): import("rxjs").Observable<Event>;
    useListener(eventSpec: EventMatcher, handler: Listener<Event, any>, options?: ListenerConfigWithDeps, deps?: any[] | undefined): void;
    useFilter(eventSpec: EventMatcher, filter: Filter<Event>, deps?: Array<any>): void;
};
/** Allows a component to attach (maybe async) consequences
 * to event patterns: strings, regexes, or function predicates.
 * Handlers return Observables for flexible scheduling/cancelation.
 * Cleans up when the component is unmounted.
 * @argument eventSpec - the criteria for the event to listen for
 * @argument handler - the function to be run upon the event, after all filters
 * @argument options - Specify the output events to be triggered in this object.
 * @returns Array of [trigger, unsub] - where trigger fires ANY event, and unsub cancels any inflight events (and is called automatcially on unmount).
 */
export declare const useListener: (eventSpec: EventMatcher, handler: Listener<Event, any>, options?: ListenerConfigWithDeps) => [(type: string, payload: any) => void, () => Subscription];
/** Allows a component to intercept and run synchronous
   consequences, alter events, or throw errors to cancel the processing
   * @argument eventSpec - the criteria for the event to listen for
   * @argument filter - the function to be run synchronously upon the event before all listeners
   * @returns Array of [trigger, unsub] - where trigger fires the event, and unsub stops filtering, (and is called automatcially on unmount)
   */
export declare const useFilter: (eventSpec: EventMatcher, filter: Filter<Event>) => ((type: string, payload?: any) => void)[];
